<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:xsd="http://www.w3.org/2001/XMLSchema">
	<xsl:param name="biproVersion" select="'2.1.0'"/>
	<xsl:param name="biproRevision" select="'22.06.2009'"/>
	<xsl:param name="pathToXMLSchemas" select="''"/><!-- z. B. 'D:/Schemas/' -->
	<xsl:output method="xml" indent="yes"/>
	
	<!--
	Dieses Stylesheet wird auf XML Schemata angewendet, die durch Enterprise Architect erzeugt wurden.
	Das Ergebnis einer Transformation mit diesem XSL ist ein neues XML-Schema, dass dem Releasestand 2.1.x
	der BiPRO-Normen entspricht.

	CHANGE.LOG
	2008-08-13	Version 1.05: BiPRO-Schemata müssen qualifizierte Elemente und Attribute in XML-Instanzen erzwingen
	2008-08-12	Version 1.04: Objekte, die nicht abstrakt sind, müssen mit abstract="false" gekennzeichnet werden
	2008-07-30	Version 1.03: Erweiterbare Aufzählungstypen werden nach neuem Modell (Verzicht auf Redefine) umgesetzt in komplexe Typen
	2008-07-09	Version 1.02: Attribute nillable="true|false" und mixed="true|false" entfernt
	2008-07-08	Version 1.01: Berücksichtung vom xmlmime-Namespace
	2008-07-08	Version 1.01: Berücksichtigung der neuen globalen Nachrichten-Elemente
	2008-06-03	Version 1.00: Veröffentlichung
	-->

	<xsl:template match="/">
		<xsl:comment>BiPRO R<xsl:value-of select="$biproVersion"/> - Revision <xsl:value-of select="$biproRevision"/></xsl:comment>
		<xsl:apply-templates/>
	</xsl:template>
	
	
	<!-- BiPRO Attribute bipro:Name und bipro:Version anhand des Ziel-Namensraumes einfügen -->
	<xsl:template match="xsd:schema">
		<xsl:copy>
			<xsl:attribute name="targetNamespace"><xsl:value-of select="@targetNamespace"/></xsl:attribute>
			<xsl:attribute name="Name" namespace="http://www.bipro.net/namespace">bipro-<xsl:value-of select="substring-after(@targetNamespace, 'http://www.bipro.net/namespace/')"/>-<xsl:value-of select="$biproVersion"/>.xsd</xsl:attribute>
			<xsl:attribute name="Version" namespace="http://www.bipro.net/namespace"><xsl:value-of select="$biproVersion"/></xsl:attribute>
			<xsl:attribute name="elementFormDefault">qualified</xsl:attribute>
			<xsl:attribute name="attributeFormDefault">qualified</xsl:attribute>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	
	
	<!-- Importanweisung um Pfad zur Schema-Datei ergänzen -->
	<xsl:template match="xsd:import">
		<xsl:choose>
			<xsl:when test="@namespace='http://www.w3.org/2004/11/xmlmime'">
				<xsd:import namespace="http://www.w3.org/2004/11/xmlmime" schemaLocation="{$pathToXMLSchemas}xmlmime-2004-11.xsd"/>
			</xsl:when>
			<xsl:otherwise>
				<xsd:import namespace="{@namespace}" schemaLocation="{$pathToXMLSchemas}bipro-{substring-after(@namespace, 'http://www.bipro.net/namespace/')}-{$biproVersion}.xsd"/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
	<!-- Namenskonventionen bei Objekten berücksichtigen CT_Name (nur bei Objekte der ersten Ebene) -->
	<xsl:template match="xsd:complexType[boolean(@name)]">
		<xsl:copy>
			<xsl:copy-of select="@*[name()!='mixed']"/>
			<xsl:attribute name="name">CT_<xsl:value-of select="@name"/></xsl:attribute>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	

	<!-- Namenskonventionen bei Referenzen auf Objekten berücksichtigen CT_Name (nicht bei Elementen auf erster Ebene) -->
	<xsl:template match="xsd:complexType//xsd:element">
		<xsl:copy>
			<xsl:copy-of select="@*[name()!='nillable']"/>
			<xsl:attribute name="type">
				<xsl:choose>
					<xsl:when test="contains(@type, 'ST')"><xsl:value-of select="@type"/></xsl:when>
					<xsl:otherwise>
						<xsl:choose>
							<xsl:when test="contains(@type, ':')"><xsl:value-of select="substring-before(@type, ':')"/>:<xsl:if test="substring-before(@type, ':')!='mime'">CT_</xsl:if><xsl:value-of select="substring-after(@type, ':')"/></xsl:when>
							<xsl:otherwise>CT_<xsl:value-of select="@type"/></xsl:otherwise>
						</xsl:choose>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<xsl:if test="@abstract!='true'"><xsl:attribute name="abstract">false</xsl:attribute></xsl:if>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>


	<!-- Namenskonventionen bei Referenzen auf Basis-Objekten berücksichtigen CT_Name -->
	<xsl:template match="xsd:extension">
		<xsl:copy>
			<xsl:attribute name="base">
				<xsl:choose>
					<xsl:when test="not(contains(@base, 'mime')) and contains(@base, ':')"><xsl:value-of select="substring-before(@base, ':')"/>:CT_<xsl:value-of select="substring-after(@base, ':')"/></xsl:when>
					<xsl:otherwise>CT_<xsl:value-of select="@base"/></xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
			<xsl:apply-templates/>
		</xsl:copy>
	</xsl:template>
	
	
	<!-- Sprachangabe in der Dokumentation hinzufügen -->
	<xsl:template match="xsd:documentation">
		<xsd:documentation xml:lang="de"><xsl:value-of select="."/></xsd:documentation>
	</xsl:template>
		

	<!-- Der Grundtyp für die Erweiterungen innerhalb der STE_xxx entfernen -->
	<xsl:template match="xsd:simpleType[contains(@name, 'Erweitert')]"/>


	<!-- Unions bei erweiterbaren Aufzählungstypen bestehen aus XxxBasis und XxxErweitert ersetzen -->
	<xsl:template match="xsd:simpleType[xsd:union]">
		<xsd:complexType name="{@name}" abstract="true">
			<xsd:simpleContent>
				<xsd:extension base="xsd:string"/>
			</xsd:simpleContent>
		</xsd:complexType>
	</xsl:template>

	
	<!-- STE_NameBasis umsetzen als komexer Typ -->
	<xsl:template match="xsd:simpleType[contains(@name, 'Basis')]">
		<xsd:complexType name="{@name}">
			<xsd:simpleContent>
				<xsd:restriction base="daten:{substring-before(@name, 'Basis')}">
					<xsl:copy-of select="xsd:restriction/*"/>
				</xsd:restriction>
			</xsd:simpleContent>
		</xsd:complexType>		
	</xsl:template>	
	
	
	<xsl:template match="@*|node()|comment()" priority="-1">
		<xsl:copy>
			<xsl:apply-templates select="@*|node()|comment()"/>
		</xsl:copy>
	</xsl:template>

</xsl:stylesheet>
